/*
             LUFA Library
     Copyright (C) Dean Camera, 2014.

  dean [at] fourwalledcubicle [dot] com
           www.lufa-lib.org
*/

/*
  Copyright 2014  Dean Camera (dean [at] fourwalledcubicle [dot] com)

  Permission to use, copy, modify, distribute, and sell this
  software and its documentation for any purpose is hereby granted
  without fee, provided that the above copyright notice appear in
  all copies and that both that the copyright notice and this
  permission notice and warranty disclaimer appear in supporting
  documentation, and that the name of the author not be used in
  advertising or publicity pertaining to distribution of the
  software without specific, written prior permission.

  The author disclaims all warranties with regard to this
  software, including all implied warranties of merchantability
  and fitness.  In no event shall the author be liable for any
  special, indirect or consequential damages or any damages
  whatsoever resulting from loss of use, data or profits, whether
  in an action of contract, negligence or other tortious action,
  arising out of or in connection with the use or performance of
  this software.
*/

/** \file
 *
 *  Main source file for the KeyboardMouse demo. This file contains the main tasks of
 *  the demo and is responsible for the initial application hardware configuration.
 */

#include "KeyboardMouse.h"
#include "psx.c"
#include <stdlib.h>     /* abs */

#define FLASH_SIZE_BYTES 32768
#ifndef BOOTLOADER_SEC_SIZE_BYTES
#define BOOTLOADER_SEC_SIZE_BYTES                               (0x1000)
#endif

#define FLASH_SIZE_BYTES 32768
#ifndef BOOTLOADER_SEC_SIZE_BYTES
#define BOOTLOADER_SEC_SIZE_BYTES                               (0x1000)
#endif

int purple=0;
int purplecount=0;
int bn=0;
int mode=0;


int myScale[]={
-127,-126,-125,-124,-122,-121,-120,-119,-117,-116,-115,-114,-112,-111,-110,-109,-107,-106,
-105,-104,-102,-101,-100,-99,-97,-96,-95,-94,-92,-91,-90,-89,-87,-86,-85,-84,-82,-81,-80,
-79,-77,-76,-75,-74,-72,-71,-70,-69,-67,-66,-65,-64,-62,-61,-60,-59,-57,-56,-55,-54,-52,
-51,-50,-49,-47,-46,-45,-44,-42,-41,-40,-39,-37,-36,-35,-34,-32,-31,-30,-29,-27,-26,-25,-24,
-22,-21,-20,-19,-17,-16,-15,-14,-12,-11,-10,-9,-7,-6,-5,-4,-2,-1,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,2,3,5,6,7,8,
10,11,12,13,15,16,17,18,20,21,22,23,25,26,27,28,30,31,32,33,35,36,37,38,40,41,42,43,45,46,47,
48,50,51,52,53,55,56,57,58,60,61,62,63,65,66,67,68,70,71,72,73,75,76,77,78,80,81,82,83,84,86,
87,88,89,91,92,93,94,96,97,98,99,101,102,103,104,106,107,108,109,111,112,113,114,116,117,118,
119,121,122,123,124,126,127
};


uint32_t Boot_Key ATTR_NO_INIT;

#define MAGIC_BOOT_KEY            0xDC42ACCA
#define BOOTLOADER_START_ADDRESS  (FLASH_SIZE_BYTES - BOOTLOADER_SEC_SIZE_BYTES)

  
  void Bootloader_Jump_Check(void) ATTR_INIT_SECTION(3);
  void Bootloader_Jump_Check(void)
  {
      // If the reset source was the bootloader and the key is correct, clear it and jump to the bootloader
      if ((MCUSR & (1<<WDRF)) && (Boot_Key == MAGIC_BOOT_KEY))
      {
          Boot_Key = 0;
          ((void (*)(void))BOOTLOADER_START_ADDRESS)(); 
      }
  }

void USB_ShutDown(void)
{
        USB_INT_DisableAllInterrupts();
        USB_INT_ClearAllInterrupts();

        USB_Detach();
        USB_Controller_Disable();
        
        if (!(USB_Options & USB_OPT_MANUAL_PLL))
          USB_PLL_Off();
        
        USB_REG_Off();

        #if defined(USB_SERIES_4_AVR) || defined(USB_SERIES_6_AVR) || defined(USB_SERIES_7_AVR)
        USB_OTGPAD_Off();
        #endif

        #if defined(USB_CAN_BE_BOTH)
        USB_CurrentMode = USB_MODE_NONE;
        #endif

        USB_IsInitialized = false;
}

  void Jump_To_Bootloader(void)
  {
      // If USB is used, detach from the bus
      USB_ShutDown();

      // Disable all interrupts
      cli();

      // Wait two seconds for the USB detachment to register on the host
      for (uint8_t i = 0; i < 128; i++)
        _delay_ms(4);

      // Set the bootloader key to the magic value and force a reset
      Boot_Key = MAGIC_BOOT_KEY;
      wdt_enable(WDTO_250MS);
      for (;;); 
  }

/** Buffer to hold the previously generated Keyboard HID report, for comparison purposes inside the HID class driver. */
static uint8_t PrevjoystickHIDReportBuffer[sizeof(USB_JoystickReport_Data_t)];

/** Buffer to hold the previously generated Mouse HID report, for comparison purposes inside the HID class driver. */
static uint8_t PrevMouseHIDReportBuffer[sizeof(USB_Mouse_Data_t)];

USB_ClassInfo_HID_Device_t Joystick_HID_Interface =
	{
		.Config =
			{
				.InterfaceNumber              = 0,
				.ReportINEndpoint             =
					{
						.Address              = JOYSTICK_IN_EPADDR,
						.Size                 = HID_EPSIZE,
						.Banks                = 1,
					},
				.PrevReportINBuffer           = PrevjoystickHIDReportBuffer,
				.PrevReportINBufferSize       = sizeof(PrevjoystickHIDReportBuffer),
			},
	};

/** LUFA HID Class driver interface configuration and state information. This structure is
 *  passed to all HID Class driver functions, so that multiple instances of the same class
 *  within a device can be differentiated from one another. This is for the mouse HID
 *  interface within the device.
 */
USB_ClassInfo_HID_Device_t Mouse_HID_Interface =
	{
		.Config =
			{
				.InterfaceNumber              = INTERFACE_ID_Mouse,
				.ReportINEndpoint             =
					{
						.Address              = MOUSE_IN_EPADDR,
						.Size                 = HID_EPSIZE,
						.Banks                = 1,
					},
				.PrevReportINBuffer           = PrevMouseHIDReportBuffer,
				.PrevReportINBufferSize       = sizeof(PrevMouseHIDReportBuffer),
			},
	};


/** Main program entry point. This routine contains the overall program flow, including initial
 *  setup of all components and the main program loop.
 */
int main(void)
{
	SetupHardware();

	LEDs_SetAllLEDs(LEDMASK_USB_NOTREADY);
	GlobalInterruptEnable();

	psx_init(&PORTD, 3,	&PORTD, 2,	&PORTD, 5,	&PORTD, 6);

    	
// Neil, don't change these, you broke them once before :P
	// 0 = input, 1 = output
	DDRB =  0b00000101;
	PORTB = 0b00001010;
	//        76543210
    DDRC =  0b11110000;  // PC4..PC7 are o/p 
    PORTC = 0b11010000;  // PC4 = 1.  PC5..PC7 are 0. RGB all lit 

	int R = 1;
	int G = 0;
	int B = 0;

	int oldmenubutton=0;

	for (;;)
	{
	
		PORTC = (!G<<7) | (!B<<6) | (!R<<5) | (1<<4);
	
		psx_read_gamepad();

		purple = !(PINB & (1<<PB1));
		int modebutton = !(PINB & (1<<PB3));
		
		if(modebutton == 1 && oldmenubutton == 0){
			mode++;
			if(mode == 3)mode=0;
		}
		oldmenubutton = modebutton;
		
		
		if(mode == 0){ R=1; G=0; B=0;}
		if(mode == 1){ R=0; G=1; B=0;}
		if(mode == 2){ R=0; G=0; B=1;}
		
		if(purple){
		    PORTC = 0b01010000;  // PC4 = 1.  PC5..PC7 are 0. RGB all lit 
			purplecount++;
		}else{
			purplecount=0;
		}


		// purple button && psx_button(PSB_CROSS) && psx_button(PSB_SQUARE) && psx_button(PSB_TRIANGLE) && psx_button(PSB_CIRCLE)
		if(purple == 1 && purplecount > 80 ){
			purple = 0;
			Jump_To_Bootloader();
		}


	
		HID_Device_USBTask(&Joystick_HID_Interface);
		HID_Device_USBTask(&Mouse_HID_Interface);
		USB_USBTask();
	}
}

/** Configures the board hardware and chip peripherals for the demo's functionality. */
void SetupHardware()
{
#if (ARCH == ARCH_AVR8)
	/* Disable watchdog if enabled by bootloader/fuses */
	MCUSR &= ~(1 << WDRF);
	wdt_disable();

	/* Disable clock division */
//	clock_prescale_set(clock_div_1);
#elif (ARCH == ARCH_XMEGA)
	/* Start the PLL to multiply the 2MHz RC oscillator to 32MHz and switch the CPU core to run from it */
	XMEGACLK_StartPLL(CLOCK_SRC_INT_RC2MHZ, 2000000, F_CPU);
	XMEGACLK_SetCPUClockSource(CLOCK_SRC_PLL);

	/* Start the 32MHz internal RC oscillator and start the DFLL to increase it to 48MHz using the USB SOF as a reference */
	XMEGACLK_StartInternalOscillator(CLOCK_SRC_INT_RC32MHZ);
	XMEGACLK_StartDFLL(CLOCK_SRC_INT_RC32MHZ, DFLL_REF_INT_USBSOF, F_USB);

	PMIC.CTRL = PMIC_LOLVLEN_bm | PMIC_MEDLVLEN_bm | PMIC_HILVLEN_bm;
#endif

	/* Hardware Initialization */
	Joystick_Init();
	LEDs_Init();
	USB_Init();
}

/** Event handler for the library USB Connection event. */
void EVENT_USB_Device_Connect(void)
{
    LEDs_SetAllLEDs(LEDMASK_USB_ENUMERATING);
}

/** Event handler for the library USB Disconnection event. */
void EVENT_USB_Device_Disconnect(void)
{
    LEDs_SetAllLEDs(LEDMASK_USB_NOTREADY);
}

/** Event handler for the library USB Configuration Changed event. */
void EVENT_USB_Device_ConfigurationChanged(void)
{
	bool ConfigSuccess = true;

	ConfigSuccess &= HID_Device_ConfigureEndpoints(&Joystick_HID_Interface);
	ConfigSuccess &= HID_Device_ConfigureEndpoints(&Mouse_HID_Interface);

	USB_Device_EnableSOFEvents();

	LEDs_SetAllLEDs(ConfigSuccess ? LEDMASK_USB_READY : LEDMASK_USB_ERROR);
}

/** Event handler for the library USB Control Request reception event. */
void EVENT_USB_Device_ControlRequest(void)
{
	HID_Device_ProcessControlRequest(&Joystick_HID_Interface);
	HID_Device_ProcessControlRequest(&Mouse_HID_Interface);
}

/** Event handler for the USB device Start Of Frame event. */
void EVENT_USB_Device_StartOfFrame(void)
{
	HID_Device_MillisecondElapsed(&Joystick_HID_Interface);
	HID_Device_MillisecondElapsed(&Mouse_HID_Interface);
}

/** HID class driver callback function for the creation of HID reports to the host.
 *
 *  \param[in]     HIDInterfaceInfo  Pointer to the HID class interface configuration structure being referenced
 *  \param[in,out] ReportID    Report ID requested by the host if non-zero, otherwise callback should set to the generated report ID
 *  \param[in]     ReportType  Type of the report to create, either HID_REPORT_ITEM_In or HID_REPORT_ITEM_Feature
 *  \param[out]    ReportData  Pointer to a buffer where the created report should be stored
 *  \param[out]    ReportSize  Number of bytes written in the report (or zero if no report is to be sent)
 *
 *  \return Boolean \c true to force the sending of the report, \c false to let the library determine if it needs to be sent
 */
bool CALLBACK_HID_Device_CreateHIDReport(USB_ClassInfo_HID_Device_t* const HIDInterfaceInfo,
                                         uint8_t* const ReportID,
                                         const uint8_t ReportType,
                                         void* ReportData,
                                         uint16_t* const ReportSize)
{

	/* Determine which interface must have its report generated */
	if (HIDInterfaceInfo == &Joystick_HID_Interface)
	{

		USB_JoystickReport_Data_t* JoystickReport = (USB_JoystickReport_Data_t*)ReportData;

	int lx = psx_stick(PSS_LX); // -128
	int ly = psx_stick(PSS_LY);
	int rx = psx_stick(PSS_RX);
	int ry = psx_stick(PSS_RY);

	if (purple)							JoystickReport->Button |= (1 << 14); // menu

	if(mode==0){
		JoystickReport->X = myScale[lx];
		JoystickReport->Y = myScale[ly];
		JoystickReport->RX = myScale[rx];
		JoystickReport->RY = myScale[ry];

		if (psx_button(PSB_CROSS))			JoystickReport->Button |= (1 << 0); // O
		if (psx_button(PSB_SQUARE))			JoystickReport->Button |= (1 << 1); // U
		if (psx_button(PSB_TRIANGLE))		JoystickReport->Button |= (1 << 2); // Y
		if (psx_button(PSB_CIRCLE))			JoystickReport->Button |= (1 << 3); // A
	
		if (psx_button(PSB_L1))				JoystickReport->Button |= (1 << 4); //
		if (psx_button(PSB_R1))				JoystickReport->Button |= (1 << 5); // 
		if (psx_button(PSB_L3))				JoystickReport->Button |= (1 << 6); // 
		if (psx_button(PSB_R3))				JoystickReport->Button |= (1 << 7); // 

		if (psx_button(PSB_PAD_UP))			JoystickReport->Button |= (1<<8);
		if (psx_button(PSB_PAD_DOWN))		JoystickReport->Button |= (1<<9);
		if (psx_button(PSB_PAD_LEFT))		JoystickReport->Button |= (1<<10);
		if (psx_button(PSB_PAD_RIGHT))		JoystickReport->Button |= (1<<11);

		if (psx_button(PSB_L2)){
			JoystickReport->Button |= (1 << 12);
			JoystickReport->Z = 127;
		}
		if (psx_button(PSB_R2)){
			JoystickReport->Button |= (1 << 13);
			JoystickReport->RZ = 127;
		}
	
		if (psx_button(PSB_SELECT))			JoystickReport->B2 |= (1 << 0); // select
		if (psx_button(PSB_START))			JoystickReport->B2 |= (1 << 1); // start
	}// mode == 0

		*ReportSize = sizeof(USB_JoystickReport_Data_t);
		return false;
	}
	else
	{
		USB_Mouse_Data_t* Mouse = (USB_Mouse_Data_t*)ReportData;

	if(mode==1){
		if(psx_stick(PSS_LX) > 150 || psx_stick(PSS_LX) < 104)	Mouse->X = (psx_stick(PSS_LX)-127)/5;
		if(psx_stick(PSS_LY) > 150 || psx_stick(PSS_LY) < 104)	Mouse->Y = (psx_stick(PSS_LY)-127)/5;
		if(psx_button(PSB_CROSS) || psx_button(PSB_SQUARE)) Mouse->Button = (1 << 0);
		if(psx_button(PSB_CIRCLE) || psx_button(PSB_TRIANGLE)) Mouse->Button = (1 << 1);
	} // mode == 1

	if(mode==2){
		if(psx_stick(PSS_RX) > 150 || psx_stick(PSS_RX) < 104)	Mouse->X = (psx_stick(PSS_RX)-127)/5;
		if(psx_stick(PSS_RY) > 150 || psx_stick(PSS_RY) < 104)	Mouse->Y = (psx_stick(PSS_RY)-127)/5;
		if(psx_button(PSB_CROSS) || psx_button(PSB_SQUARE)) Mouse->Button = (1 << 0);
		if(psx_button(PSB_CIRCLE) || psx_button(PSB_TRIANGLE)) Mouse->Button = (1 << 1);
	} // mode == 1


		*ReportSize = sizeof(USB_Mouse_Data_t);
		return true;
	}
}

/** HID class driver callback function for the processing of HID reports from the host.
 *
 *  \param[in] HIDInterfaceInfo  Pointer to the HID class interface configuration structure being referenced
 *  \param[in] ReportID    Report ID of the received report from the host
 *  \param[in] ReportType  The type of report that the host has sent, either HID_REPORT_ITEM_Out or HID_REPORT_ITEM_Feature
 *  \param[in] ReportData  Pointer to a buffer where the received report has been stored
 *  \param[in] ReportSize  Size in bytes of the received HID report
 */
void CALLBACK_HID_Device_ProcessHIDReport(USB_ClassInfo_HID_Device_t* const HIDInterfaceInfo,
                                          const uint8_t ReportID,
                                          const uint8_t ReportType,
                                          const void* ReportData,
                                          const uint16_t ReportSize)
{
}

